﻿using System;
using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 그래프
    /// </summary>
    /// <typeparam name="T">항목 타입</typeparam>
    public class Graph<T>
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 이웃 딕셔너리 - AdjacencyDictionary

        /// <summary>
        /// 이웃 딕셔너리
        /// </summary>
        public Dictionary<T, HashSet<T>> AdjacencyDictionary { get; } = new Dictionary<T, HashSet<T>>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Graph()

        /// <summary>
        /// 생성자
        /// </summary>
        public Graph()
        {
        }

        #endregion
        #region 생성자 - Graph(vertexEnumerable, edgeEnumerable)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="vertexEnumerable">정점 열거형</param>
        /// <param name="edgeEnumerable">엣지 열거형</param>
        public Graph(IEnumerable<T> vertexEnumerable, IEnumerable<Tuple<T,T>> edgeEnumerable)
        {
            foreach(T vertex in vertexEnumerable)
            {
                AddVertex(vertex);
            }

            foreach(Tuple<T, T> edge in edgeEnumerable)
            {
                AddEdge(edge);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 정점 추가하기 - AddVertex(vertex)

        /// <summary>
        /// 정점 추가하기
        /// </summary>
        /// <param name="vertex">정점</param>
        public void AddVertex(T vertex)
        {
            AdjacencyDictionary[vertex] = new HashSet<T>();
        }

        #endregion
        #region 엣지 추가하기 - AddEdge(edge)

        /// <summary>
        /// 엣지 추가하기
        /// </summary>
        /// <param name="edge">엣지</param>
        public void AddEdge(Tuple<T,T> edge)
        {
            if(AdjacencyDictionary.ContainsKey(edge.Item1) && AdjacencyDictionary.ContainsKey(edge.Item2))
            {
                AdjacencyDictionary[edge.Item1].Add(edge.Item2);
                AdjacencyDictionary[edge.Item2].Add(edge.Item1);
            }
        }

        #endregion
    }
}