﻿using System.Collections.Generic;

namespace TestProject
{
    /// <summary>
    /// 그래프 헬퍼
    /// </summary>
    public static class GraphHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 깊이 우선 탐색하기 - DepthFirstSearch<T>(graph, startItem)

        /// <summary>
        /// 깊이 우선 탐색하기
        /// </summary>
        /// <typeparam name="T">항목 타입</typeparam>
        /// <param name="graph">그래프</param>
        /// <param name="startItem">시작 항목</param>
        /// <returns>깊이-우선 탐색 결과</returns>
        public static HashSet<T> DepthFirstSearch<T>(Graph<T> graph, T startItem)
        {
            HashSet<T> visitedVertexHashSet = new HashSet<T>();

            if(!graph.AdjacencyDictionary.ContainsKey(startItem))
            {
                return visitedVertexHashSet;
            }
                
            Stack<T> stack = new Stack<T>();

            stack.Push(startItem);

            while(stack.Count > 0)
            {
                T vertex = stack.Pop();

                if(visitedVertexHashSet.Contains(vertex))
                {
                    continue;
                }

                visitedVertexHashSet.Add(vertex);

                foreach(T neighborVertex in graph.AdjacencyDictionary[vertex])
                {
                    if(!visitedVertexHashSet.Contains(neighborVertex))
                    {
                        stack.Push(neighborVertex);
                    }
                }
            }

            return visitedVertexHashSet;
        }

        #endregion
    }
}